import Foundation

// Model & JSON

struct Task {
    let name: String
    let completed: Bool
    let position: Int
    let project: String?
    
    static func create(name: String)(completed: Bool)(position: Int)(project: String?) -> Task {
        return Task(name: name, completed: completed, position: position, project: project)
    }
}

let json = ["name": "Do this", "completed": false, "position": 10]

// Functional operators

infix operator >>> { associativity left precedence 150 }

func >>> <A, B>(a: A?, f: A -> B?) -> B? {
    if let a = a {
        return f(a)
    } else {
        return nil
    }
}

infix operator <%> { associativity left }

func <%> <A, B>(f: A -> B, a: A?) -> B? {
    if let a = a {
        return f(a)
    } else {
        return nil
    }
}

infix operator <*> { associativity left }

func <*> <A, B>(f: (A -> B)?, a: A?) -> B? {
    if let f = f, a = a {
        return f(a)
    } else {
        return nil
    }
}

// Functional helpers

func pure<T>(x: T) -> T? {
    return .Some(x)
}

// Casting helpers

func autocast<T>(x: AnyObject) -> T? {
    return x as? T
}

// JSON parsing operators

infix operator <| { associativity left precedence 150 }

func <| <T>(json: [String: AnyObject], key: String) -> T? {
    return json[key] >>> autocast
}

infix operator <|? { associativity left precedence 150 }

func <|? <T>(json: [String: AnyObject], key: String) -> T?? {
    return pure(json[key] >>> autocast)
}

// Decoding

let task = Task.create
    <%> json <|  "name"
    <*> json <|  "completed"
    <*> json <|  "position"
    <*> json <|? "project"
